# Copyright (c) 2024 Alethea Katherine Flowers.
# Published under the standard MIT License.
# Full text available at: https://opensource.org/licenses/MIT

__version__ = "2024.11.29"
__author__ = "Alethea Katherine Flowers"
__author_email__ = "me@thea.codes"
